#include "include/SimpleMesh.h"
#include "ply2mesh.h"
#include <stdio.h>
#include <list>
#include <stack>
#include <assert.h>
#include <iostream>



using namespace std;



float			oldcenter[3];															//!!!Should go in some header/datastruct
float			newradius;
static  float	center[3] = { 0.0, 0.0, 0.0 };											//ALEX: This was declared global in the old skel_main.cpp but apparently never set/read except here below..





SimpleMesh::SimpleMesh(): has_vertex_normals(false)
{  }


SimpleMesh::~SimpleMesh()
{
}

int SimpleMesh::readPly(const char* fname)
{
  return ::read_ply(fname,points,faces);
}

int SimpleMesh::readDMS(const char* fname)
{
    FILE* fp = fopen(fname,"rb");
    if (!fp) return 1;

    float xyz[3];

    while (fread(xyz,sizeof(float),3,fp)==3)
    {
       points.push_back(Point3d(xyz[0],xyz[1],xyz[2]));
    }

    fclose(fp);
    return 0;
}


void SimpleMesh::center(Point3d& p)						//Compute barycenter of the mesh
{
	p = Point3d(0,0,0);

	for(int i=0,NF=faces.size();i<NF;++i)
	{
		const face&   f = faces[i];
		const float   a = f.area(points);
		const Point3d c = f.center(points);
		p.x += c.x*a;
		p.y += c.y*a;
		p.z += c.z*a;
	}
}


void SimpleMesh::clearFaces()
{
  faces.clear();
  has_vertex_normals = false;
}

float SimpleMesh::scaleDown()
{
    int i;
    float minmax[6];
    float radius = 2.0;

    int nverts=(int)points.size();

    // Determine model extents.
    minmax[0] = 99999999.99f;    minmax[1] = -99999999.99f;
    minmax[2] = 99999999.99f;    minmax[3] = -99999999.99f;
    minmax[4] = 99999999.99f;    minmax[5] = -99999999.99f;
    for ( i = 0; i < nverts; i++ )
        {
        float *v = (float *)(&points[i]);
        if ( v[0] < minmax[0] ) minmax[0] = v[0];
        if ( v[0] > minmax[1] ) minmax[1] = v[0];
        if ( v[1] < minmax[2] ) minmax[2] = v[1];
        if ( v[1] > minmax[3] ) minmax[3] = v[1];
        if ( v[2] < minmax[4] ) minmax[4] = v[2];
        if ( v[2] > minmax[5] ) minmax[5] = v[2];
        }

    // Scale to shift to [-radius,radius]
    float rad[3];
    rad[0] = 0.5f*(minmax[1]-minmax[0]);
    rad[1] = 0.5f*(minmax[3]-minmax[2]);
    rad[2] = 0.5f*(minmax[5]-minmax[4]);

    oldcenter[0] = minmax[0] + rad[0];
    oldcenter[1] = minmax[2] + rad[1];
    oldcenter[2] = minmax[4] + rad[2];

    float oldradius;
    oldradius = sqrtf( rad[0]*rad[0] + rad[1]*rad[1] + rad[2]*rad[2] );
    newradius = radius / oldradius;

    for ( i = 0; i < nverts; i++ )
    {
	float *v = (float *)(&points[i]);
	v[0] = ::center[0] + newradius*(v[0]-oldcenter[0]);
	v[1] = ::center[1] + newradius*(v[1]-oldcenter[1]);
	v[2] = ::center[2] + newradius*(v[2]-oldcenter[2]);
    }

    return newradius;
}

void SimpleMesh::setNormals(bool flip,bool area_weighting)								//Compute vertex normals from face normals with or w/o area weighting
{																					//If 'flip' then flip normals
  if (!faces.size()) return;


  int nVertex =(int)points.size(), nFaces=(int)faces.size();
  vector<float> normals(3*nVertex, 0.0f);											//Initialize vertex normals

  for(int i=0;i<nFaces;i++)
  {
	face& f = faces[i];
    f.SetNormal(points);															//Compute face normal

	float area = (area_weighting)? f.area(points) : 1;								//Area-weighting of normals (or not)
	if (area<1.0e-8) area = 1.0e-8;
    if (flip) f.flipNormal();														//Swap normal if so desired

	normals[3*f.index[0]]   +=area*f.normal.x;
    normals[3*f.index[0]+1] +=area*f.normal.y;
    normals[3*f.index[0]+2] +=area*f.normal.z;

    normals[3*f.index[1]]  +=area*f.normal.x;
    normals[3*f.index[1]+1]+=area*f.normal.y;
    normals[3*f.index[1]+2]+=area*f.normal.z;

    normals[3*f.index[2]]  +=area*f.normal.x;
    normals[3*f.index[2]+1]+=area*f.normal.y;
    normals[3*f.index[2]+2]+=area*f.normal.z;
  }

  for(int i=0;i<nVertex;i++)
  {
    float mag=sqrt(normals[3*i]*normals[3*i]+ normals[3*i+1]*normals[3*i+1]+normals[3*i+2]*normals[3*i+2]);
	if (mag<1.0e-8) mag=1.0e-8;
	Point3d n(normals[3*i]/mag,normals[3*i+1]/mag,normals[3*i+2]/mag); n.normalize();
    points[i].n[0]=n[0];
    points[i].n[1]=n[1];
    points[i].n[2]=n[2];
  }

  has_vertex_normals = true;
}



float SimpleMesh::signedVolume(int face_id) const
//signed volume of tetrahedron with vertices (p1,p2,p3,origin)
{
	const face&     f = faces[face_id];
	const Point3d& p1 = points[f.index[0]], &p2 = points[f.index[1]], &p3 = points[f.index[2]];

	float v321 = p3.x*p2.y*p1.z;
	float v231 = p2.x*p3.y*p1.z;
	float v312 = p3.x*p1.y*p2.z;
	float v132 = p1.x*p3.y*p2.z;
	float v213 = p2.x*p1.y*p3.z;
	float v123 = p1.x*p2.y*p3.z;

	return (-v321 + v231 + v312 - v132 - v213 + v123)/6.0f;
}
