#pragma once

// This file:			Implementation of a (triangular) face for a mesh
//
//

#include "Point3d.h"
#include <vector>
#include <math.h>


using namespace std;

class face			//Triangular face using vertex indexes
{
public:
  int	  	index[3];
  Point3d 	normal;
  int	  	oid;

                face(int i=0, int j=0, int k=0): oid(-1) { index[0]=i; index[1]=j; index[2]=k; }
                face(const face &f): normal(f.normal), oid(f.oid) 
                { index[0]=f.index[0]; index[1]=f.index[1]; index[2]=f.index[2]; }
  face& 	operator=(const face &f) 
                {
                  if(&f==this) return *this;
                  index[0]=f.index[0]; index[1]=f.index[1]; index[2]=f.index[2];
                  normal = f.normal;
                  oid = f.oid;
                  return *this;
                }
  int 		operator[](int i) const { return index[i]; }
  int&		operator[](int i) { return index[i]; }
  void 		SetNormal(vector<Point3d>& points, bool normalize=true)
                {
                  Point3d &v0 =  points[index[0]];
                  Point3d &v1 =  points[index[1]];
                  Point3d &v2 =  points[index[2]];
                  Point3d p1  = v2-v1, p2 = v1-v0;
                  normal.x = - p1.y*p2.z + p1.z*p2.y; 
                  normal.y = - p1.z*p2.x + p2.z*p1.x; 
                  normal.z = - p1.x*p2.y + p1.y*p2.x; 

                  if (normalize) 
                  {
                      double n = sqrt((normal.x*normal.x)+(normal.y*normal.y)+(normal.z*normal.z));
                      if (n!=0)
                      {
                         normal.x/=n;
                         normal.y/=n;
                         normal.z/=n;
                      }
                  }
                }
	
  float 	area(const vector<Point3d>& p) const
                {
                  int a=index[0],b=index[1],c=index[2];
                  Point3d ab = p[b]-p[a], ac = p[c]-p[a];
                  return 0.5*ab.cross(ac).norm();
                }

  void 		flipNormal() 
                {  normal.x=-normal.x; normal.y=-normal.y; normal.z=-normal.z;  }

  int other_point(int idx1, int idx2) 
  {
    if(index[0]==idx1 && index[1]==idx2) return index[2];
    if(index[0]==idx2 && index[1]==idx1) return index[2];

    if(index[1]==idx1 && index[2]==idx2) return index[0];
    if(index[1]==idx2 && index[2]==idx1) return index[0];

    if(index[0]==idx1 && index[2]==idx2) return index[1];
    if(index[0]==idx2 && index[2]==idx1) return index[1];    

    return -1;
  }

  size_t 	operator()(const face &f) const 
                {  return f.index[0]+f.index[1]+f.index[2];  }

  bool 		operator==(const face &f) const 
                {  return ((index[0]+index[1]+index[2])==(f.index[0]+f.index[1]+f.index[2]));  }
 
  bool 		operator<(const face &f) const 
                {  return ((index[0]+index[1]+index[2])<(f.index[0]+f.index[1]+f.index[2]));  }
  
  Point3d 	center(const vector<Point3d>& p) const
                { 
                  Point3d o = p[index[0]] + p[index[1]] + p[index[2]];
	          return o/3;
                }
};  
